#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define output(directions,pin) (directions |= pin) 	// set port direction for output
#define input(directions,pin) (directions &= (~pin))	// set port direction for input
#define set(port,pin) (port |= pin) 			// set port pin
#define clear(port,pin) (port &= (~pin)) 		// clear port pin
#define pin_test(pins,pin) (pins & pin) 		// test for port pin: if button is pressed I have 0 because it is connected to GND
#define led_delay() _delay_ms(100) 			// LED delay

#define input_port PORTB
#define input_direction DDRB
#define input_pin (1 << PB2)
#define input_pins PINB

#define led_port PORTA
#define led_direction DDRA
#define led_pin (1 << PA7)


ISR(INT0_vect) { /* run every time there is a change on button */

	if (!pin_test(input_pins, input_pin)) {
//		led_port |= led_pin;
		led_direction ^= led_pin;
	}
	else {
//		led_port &= ~led_pin;
	}
}

void initInterrupt0(void) {
	GIMSK |= (1 << INT0);	// enable INT0. (enable external pin interrupt (pag.50 datasheet))
	MCUCR |= (1 << ISC00);	// trigger when button changes (pag. 50 datasheet)
	sei(); 			// set global interrupt when enable it. (enable SREG. we use I bit)
}


int main(void) {

   // set clock divider to /1
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);

   initInterrupt0();

   // initialize LED pin
   clear(led_port, led_pin);
   output(led_direction, led_pin);
//   input(input_direction, input_pin); // OPTIONAL!

   while (1) {

//	led_port &= ~led_pin;
//	led_port ^= ~led_pin;
	set(led_port, led_pin);
	led_delay();
	clear(led_port, led_pin);
	led_delay();

   }
}
